<?php 

/**
 * Gets an array of capabilities according to each user role.  Each role will return its caps, 
 * which are then added to the overall $capabilities array.
 *
 * Note that if no role has the capability, it technically no longer exists.  Since this could be 
 * a problem with folks accidentally deleting the default WordPress capabilities, the 
 * members_default_capabilities() will return those all the defaults.
 *
 * @since 0.1
 * @return $capabilities array All the capabilities of all the user roles.
 * @global $wp_roles array Holds all the roles for the installation.
 */
function bp_restrictgroups_admin_get_role_capabilities() {
	global $wp_roles;

	$capabilities = array();

	/* Loop through each role object because we need to get the caps. */
	foreach ( $wp_roles->role_objects as $key => $role ) {

		/* Roles without capabilities will cause an error, so we need to check if $role->capabilities is an array. */
		if ( is_array( $role->capabilities ) ) {

			/* Loop through the role's capabilities and add them to the $capabilities array. */
			foreach ( $role->capabilities as $cap => $grant )
				$capabilities[$cap] = $cap;
		}
	}

	/* Return the capabilities array. */
	return $capabilities;
}

/**
 * Checks if a specific capability has been given to at least one role. If it has,
 * return true. Else, return false.
 *
 * @since 0.1
 * @uses members_get_role_capabilities() Checks for capability in array of role caps.
 * @param $cap string Name of the capability to check for.
 * @return true|false bool Whether the capability has been given to a role.
 */
function bp_restrictgroups_admin_check_for_cap( $cap = '' ) {

	/* Without a capability, we have nothing to check for.  Just return false. */
	if ( !$cap )
		return false;

	/* Gets capabilities that are currently mapped to a role. */
	$caps = bp_restrictgroups_admin_get_role_capabilities();

	/* If the capability has been given to at least one role, return true. */
	if ( in_array( $cap, $caps ) )
		return true;

	/* If no role has been given the capability, return false. */
	return false;
}

function bp_restrictgroups_admin() {
	global $bp;

	/* If the form has been submitted and the admin referrer checks out, save the settings */
	if ( isset( $_POST['submit'] ) && check_admin_referer('bp_restrictgroups_admin') ) {
	
		//check for valid cap and update - if not keep old.
		if( isset($_POST['cap_low'] ) && !empty($_POST['cap_low']) ) {
			if ( bp_restrictgroups_admin_check_for_cap( $_POST['cap_low'] ) ) {
				update_option( 'bp_restrictgroups_cap_low', $_POST['cap_low'] );
			} else {
				echo '<div id="message" class="error"><p>Invalid capability for group creation - please see <a href="http://codex.wordpress.org/Roles_and_Capabilities#Capability_vs._Role_Table">WP Roles and Capabilities</a>.</p></div>';
			}
		} else {
			update_option( 'bp_restrictgroups_cap_low', 'edit_posts' );
			
			echo '<div id="message" class="updated fade"><p>Group creation capability was left blank - this is required - assuming \'edit_posts\'.</p></div>';
		}
	
		//check if enable a cap on the forum option
		if ( function_exists('bp_forums_is_installed_correctly') && bp_forums_is_installed_correctly() ) {
			if( isset($_POST['cap_enable_forum'] )  && !empty($_POST['cap_enable_forum']) ) {
				if ( bp_restrictgroups_admin_check_for_cap( $_POST['cap_enable_forum'] ) ) {
					update_option( 'bp_restrictgroups_cap_enable_forum', $_POST['cap_enable_forum'] );
				} else {
					echo '<div id="message" class="error"><p>Invalid capability for group forum creation - please see <a href="http://codex.wordpress.org/Roles_and_Capabilities#Capability_vs._Role_Table">WP Roles and Capabilities</a>.</p></div>';
				}
			} else {
				//assume our default contributor
				update_option( 'bp_restrictgroups_cap_enable_forum', false );
			}
		}
	
		if ( isset($_POST['cap_enable_status_private'] ) || isset($_POST['cap_enable_status_hidden'] ) ) {
			
			$data = array();
			
			if ( isset($_POST['cap_enable_status_private']) && !empty($_POST['cap_enable_status_private']) ) {
				if ( bp_restrictgroups_admin_check_for_cap( $_POST['cap_enable_status_private'] ) ) {
					$data['private'] = $_POST['cap_enable_status_private'];
				} else {
				
					echo '<div id="message" class="error"><p>Invalid capability for private group - please see <a href="http://codex.wordpress.org/Roles_and_Capabilities#Capability_vs._Role_Table">WP Roles and Capabilities</a>.</p></div>';
				
					if ( bp_restrictgroups_enabled_status( 'private' ) )
						$data['private'] = bp_restrictgroups_enabled_status( 'private' );
				}
			}

			if ( isset($_POST['cap_enable_status_hidden']) && !empty($_POST['cap_enable_status_hidden']) ) {
				if ( bp_restrictgroups_admin_check_for_cap( $_POST['cap_enable_status_hidden'] ) ) {
					$data['hidden'] = $_POST['cap_enable_status_hidden'];
				} else {
				
					echo '<div id="message" class="error"><p>Invalid capability for hidden group - please see <a href="http://codex.wordpress.org/Roles_and_Capabilities#Capability_vs._Role_Table">WP Roles and Capabilities</a>.</p></div>';
				
					if ( bp_restrictgroups_enabled_status( 'hidden' ) ) 
						$data['hidden'] = bp_restrictgroups_enabled_status( 'hidden' );
				}
			}
			
			if ($data && count($data) > 0) {
				update_option( 'bp_restrictgroups_cap_enable_status', maybe_serialize($data) );
			} else {
				update_option( 'bp_restrictgroups_cap_enable_status', false );
			}
		}
		
		//check for valid cap and update - if not keep old.
		if( isset($_POST['demote_creator_id'] ) && !empty($_POST['demote_creator_id']) && (int)$_POST['demote_creator_id'] == 1 ) {
			update_option( 'bp_restrictgroups_demote_creator_id', true );
		} else {
			update_option( 'bp_restrictgroups_demote_creator_id', false );
		}
		
		$updated = true;
	}

if ( bp_restrictgroups_enabled_status( 'hidden' ) ) 
	$rgh = bp_restrictgroups_enabled_status( 'hidden' );
	
if ( bp_restrictgroups_enabled_status( 'private' ) )
	$rgp = bp_restrictgroups_enabled_status( 'private' );

?>	
	<div class="wrap">
		<h2><?php _e( 'Restrict Group Creation', 'bp-restrictgroups' ); ?></h2>

		<?php if ( isset($updated) ) : echo "<div id='message' class='updated fade'><p>" . __( 'Settings Updated.', 'bp-restrictgroups' ) . "</p></div>"; endif; ?>

		<form action="<?php echo site_url() . '/wp-admin/admin.php?page=bp-restrictgroups-settings' ?>" name="restrictgroups-settings-form" id="restrictgroups-settings-form" method="post">

			<table class="form-table">
				<tr valign="top">
					<th scope="row"><label for="cap_low"><?php _e( 'Create group capability', 'bp-restrictgroups' ) ?></label></th>
					<td><input type="text" name="cap_low" id="cap_low" value="<?php echo get_option( 'bp_restrictgroups_cap_low'); ?>" /></td>
				</tr>
				<?php if ( function_exists('bp_forums_is_installed_correctly') && bp_forums_is_installed_correctly() ) { ?>
				<tr>
					<th><label for="cap_enable_forum"><?php _e('Create group forum capability','bp-restrictgroups') ?></label></th>
					<td><input type="text" name="cap_enable_forum" id="cap_enable_forum" value="<?php echo get_option( 'bp_restrictgroups_cap_enable_forum'); ?>" /> <?php if (!get_option( 'bp_restrictgroups_cap_enable_forum')) echo '(<em>currently disabled</em>)'; ?></td>
				</tr>
				<?php } ?>
				<tr>
					<th><label for="cap_enable_status_private"><?php _e('Create private group capability','bp-restrictgroups') ?></label></th>
					<td><input type="text" name="cap_enable_status_private" id="cap_enable_status_private" value="<?php echo $rgp; ?>" /> <?php if (!$rgp) echo '(<em>currently disabled</em>)'; ?></td>
				</tr>

				<tr>
					<th><label for="cap_enable_status_hidden"><?php _e('Create hidden group capability','bp-restrictgroups') ?></label></th>
					<td><input type="text" name="cap_enable_status_hidden" id="cap_enable_status_hidden" value="<?php echo $rgh; ?>" /> <?php if (!$rgh) echo '(<em>currently disabled</em>)'; ?></td>
				</tr>

				<tr>
					<th><label for="demote_creator_id"><?php _e('Auto demote group creator to group mod','bp-restrictgroups') ?></label></th>
					<td><input type="checkbox" name="demote_creator_id" id="demote_creator_id" value="1"<?php if ( get_option( 'bp_restrictgroups_demote_creator_id') ) { ?> checked="checked"<?php } ?> /></td>
				</tr>
			</table>
			
			<div id='message' class="notice">
				<p>*Create group capability is required (assumes 'edit_posts' - contributor - if left blank)</p>
				<p>*If others are left blank, then those features are removed from the group creation steps (ie, no option to select forums)</p>
				<p>*Auto Demote to Mod will ensure the user can not change group settings after group creation. You may auto assign an admin or mod user_id - please check out the readme.txt
				</p>
			</div>
			
			<?php wp_nonce_field( 'bp_restrictgroups_admin' ); ?>
			
			<p class="submit"><input type="submit" name="submit" value="Save Settings"/></p>
			
		</form>
	</div>
<?php
}

?>